@@ -1,7 +1,9 @@
 
-from ..utils import get_phenolist, get_gene_tuples, pad_gene, PheWebError
-from ..conf_utils import conf
-from ..file_utils import get_filepath
+from ..load.load_utils import get_maf
+from ..utils import get_phenolist, get_gene_tuples, pad_gene, PheWebError, vep_consqeuence_category
+from .. import conf
+from .. import parse_utils
+from ..file_utils import get_filepath, get_pheno_filepath, VariantFileReader
 from .server_utils import get_variant, get_random_page, get_pheno_region
 from .autocomplete import Autocompleter
 from .auth import GoogleSignIn
@@ -26,24 +28,24 @@ bp = Blueprint('bp', __name__, template_folder='templates', static_folder='stati
 app = Flask(__name__)
 Compress(app)
 app.config['COMPRESS_LEVEL'] = 2 # Since we don't cache, faster=better
-app.config['SECRET_KEY'] = conf.SECRET_KEY if hasattr(conf, 'SECRET_KEY') else 'nonsecret key'
+app.config['SECRET_KEY'] = conf.get_secret_key()
 app.config['TEMPLATES_AUTO_RELOAD'] = True
 app.config['SEND_FILE_MAX_AGE_DEFAULT'] = 9
-if 'GOOGLE_ANALYTICS_TRACKING_ID' in conf:
-    app.config['GOOGLE_ANALYTICS_TRACKING_ID'] = conf['GOOGLE_ANALYTICS_TRACKING_ID']
-if 'SENTRY_DSN' in conf and not os.environ.get('PHEWEB_NO_SENTRY',''):
-    app.config['SENTRY_DSN'] = conf['SENTRY_DSN']
-app.config['HG_BUILD_NUMBER'] = conf['hg_build_number']
-app.config['GRCH_BUILD_NUMBER'] = conf['hg_build_number'] if conf['hg_build_number'] >= 38 else 18 + conf['hg_build_number']
+if conf.get_google_analytics_id():
+    app.config['GOOGLE_ANALYTICS_TRACKING_ID'] = conf.get_google_analytics_id()
+if conf.get_sentry_id() and not os.environ.get('PHEWEB_NO_SENTRY',''):
+    app.config['SENTRY_DSN'] = conf.get_sentry_id()
+app.config['HG_BUILD_NUMBER'] = conf.get_hg_build_number()
+app.config['GRCH_BUILD_NUMBER'] = conf.get_grch_build_number()
 app.config['PHEWEB_VERSION'] = pheweb_version
-app.config['LZJS_VERSION'] = conf['lzjs_version']  # TODO: True asset mgmt / build system in future
-app.config['LZJS_VERSION_PHEWAS'] = conf['lzjs_version_phewas']
-app.config['URLPREFIX'] = conf.urlprefix.rstrip('/')
-app.config['USE_WHITELIST'] = 'login' in conf and 'whitelist' in conf.login
-if os.path.isdir(conf.custom_templates):
+app.config['LZJS_VERSION'] = conf.get_lzjs_version()
+app.config['LZJS_VERSION_PHEWAS'] = conf.get_lzjs_version_phewas()
+app.config['URLPREFIX'] = conf.get_urlprefix()
+app.config['USE_WHITELIST'] = conf.is_login_required() and bool(conf.get_login_allowlist())
+if conf.get_custom_templates_dir():
     jinja_searchpath = getattr(app.jinja_loader, 'searchpath', None)
     if jinja_searchpath is None: raise Exception()
-    jinja_searchpath.insert(0, conf.custom_templates)
+    jinja_searchpath.insert(0, conf.get_custom_templates_dir())
 
 phenos = {pheno['phenocode']: pheno for pheno in get_phenolist()}
 
@@ -54,7 +56,7 @@ def check_auth(func):
     If they haven't, their intended destination is stored and they're sent to get authorized.
     It has to be placed AFTER @bp.route() so that it can capture `request.path`.
     """
-    if 'login' not in conf:
+    if not conf.is_login_required():
         return func
     # inspired by <https://flask-login.readthedocs.org/en/latest/_modules/flask_login.html#login_required>
     @functools.wraps(func)
@@ -64,8 +66,8 @@ def check_auth(func):
             session['original_destination'] = request.path
             return redirect(url_for('.get_authorized'))
         print('{} visited {!r}'.format(current_user.email, request.path))
-        if 'whitelist' in conf.login:
-            assert current_user.email.lower() in conf.login['whitelist'], current_user
+        if conf.get_login_allowlist():
+            assert current_user.email.lower() in conf.get_login_allowlist(), current_user
         return func(*args, **kwargs)
     return decorated_view
 
@@ -96,7 +98,7 @@ def go():
 def api_variant(query:str):
     variant = get_variant(query)
     resp = jsonify(variant)
-    if conf['allow_variant_json_cors']:
+    if conf.should_allow_variant_json_cors():
         resp.headers.add('Access-Control-Allow-Origin', '*')
     return resp
 
@@ -109,7 +111,7 @@ def variant_page(query:str):
             die("Sorry, I couldn't find the variant {}".format(query))
         return render_template('variant.html',
                                variant=variant,
-                               tooltip_lztemplate=conf.parse.tooltip_lztemplate,
+                               tooltip_lztemplate=parse_utils.tooltip_lztemplate,
         )
     except Exception as exc:
         die('Oh no, something went wrong', exc)
@@ -119,6 +121,46 @@ def variant_page(query:str):
 def api_pheno(phenocode:str):
     return send_from_directory(get_filepath('manhattan'), '{}.json'.format(phenocode))
 
+@bp.route('/api/manhattan-filtered/pheno/<phenocode>.json')
+@check_auth
+def api_pheno_filtered(phenocode):
+    pheno = phenos[phenocode]
+    indel = request.args.get('indel', '')
+    # TODO: replace these exceptions with abort(404)
+    if indel: assert indel in ['true', 'false'], indel
+    consequence_category = request.args.get('csq', '')
+    if consequence_category: assert consequence_category in ['lof', 'nonsyn'], consequence_category
+    min_maf = float(request.args['min_maf']) if request.args.get('min_maf','') else None
+    max_maf = float(request.args['max_maf']) if request.args.get('max_maf','') else None
+    chosen_variants = []  # TODO: stream straight from VariantFileReader() -> Binner() without this intermediate list
+    weakest_pval_seen = 0
+    num_variants = 0
+    filepath = get_pheno_filepath('best_of_pheno', phenocode)
+    with VariantFileReader(filepath) as vfr:
+        for v in vfr:
+            num_variants += 1
+            if v['pval'] > weakest_pval_seen: weakest_pval_seen = v['pval']
+            if indel == 'true' and len(v['ref']) == 1 and len(v['alt']) == 1: continue
+            if indel == 'false' and (len(v['ref']) != 1 or len(v['alt']) != 1): continue
+            if min_maf is not None or max_maf is not None:
+                v_maf = get_maf(v, pheno)
+                if min_maf is not None and v_maf < min_maf: continue
+                if max_maf is not None and v_maf > max_maf: continue
+            if consequence_category:
+                csq = vep_consqeuence_category.get(v.get('consequence',''), '')
+                if consequence_category == 'lof' and csq != 'lof': continue
+                if consequence_category == 'nonsyn' and not csq: continue
+            chosen_variants.append(v)
+    from pheweb.load.manhattan import Binner
+    binner = Binner()
+    for variant in chosen_variants:
+        binner.process_variant(variant)
+    manhattan_data = binner.get_result()
+    manhattan_data['weakest_pval'] = weakest_pval_seen
+    #print(f'indel={indel} maf={min_maf}-{max_maf} #chosen={len(chosen_variants)} #bins={len(manhattan_data["variant_bins"])} #unbinned={len(manhattan_data["unbinned_variants"])} weakest_pval={weakest_pval_seen}')
+    return jsonify(manhattan_data)
+
+
 @bp.route('/top_hits')
 @check_auth
 def top_hits_page():
@@ -167,13 +209,27 @@ def pheno_page(phenocode:str):
     except KeyError:
         die("Sorry, I couldn't find the pheno code {!r}".format(phenocode))
     return render_template('pheno.html',
-                           show_correlations=conf.show_correlations,
-                           pheno_correlations_pvalue_threshold=conf.pheno_correlations_pvalue_threshold,
+                           show_correlations=conf.should_show_correlations(),
+                           pheno_correlations_pvalue_threshold=conf.get_pheno_correlations_pvalue_threshold(),
+                           show_manhattan_filter_button=conf.should_show_manhattan_filter_button(),
                            phenocode=phenocode,
                            pheno=pheno,
-                           tooltip_underscoretemplate=conf.parse.tooltip_underscoretemplate,
+                           tooltip_underscoretemplate=parse_utils.tooltip_underscoretemplate,
     )
 
+@bp.route('/pheno-filter/<phenocode>')
+@check_auth
+def pheno_filter_page(phenocode):
+    try:
+        pheno = phenos[phenocode]
+    except KeyError:
+        die("Sorry, I couldn't find the pheno code {!r}".format(phenocode))
+    return render_template('pheno-filter.html',
+                           phenocode=phenocode,
+                           pheno=pheno,
+                           tooltip_underscoretemplate=parse_utils.tooltip_underscoretemplate,
+                           show_manhattan_filter_consequence=conf.should_show_manhattan_filter_consequence(),
+    )
 
 
 @bp.route('/region/<phenocode>/<region>')
@@ -187,7 +243,7 @@ def region_page(phenocode:str, region:str):
     return render_template('region.html',
                            pheno=pheno,
                            region=region,
-                           tooltip_lztemplate=conf.parse.tooltip_lztemplate,
+                           tooltip_lztemplate=parse_utils.tooltip_lztemplate,
     )
 
 @bp.route('/api/region/<phenocode>/lz-results/') # This API is easier on the LZ side.
@@ -205,7 +261,7 @@ def api_region(phenocode:str):
 @check_auth
 def api_pheno_correlations(phenocode:str):
     """Send information about phenotype correlations. This is an optional feature controlled by configuration."""
-    if not conf.show_correlations:
+    if not conf.should_show_correlations():
         return jsonify({'error': 'This PheWeb instance does not support the requested endpoint.'}), 400
 
     annotated_correl_fn = get_filepath('correlations')
@@ -274,7 +330,7 @@ def gene_phenocode_page(phenocode:str, genename:str):
                                significant_phenos=phenos_in_gene,
                                gene_symbol=genename,
                                region='{}:{}-{}'.format(chrom, start, end),
-                               tooltip_lztemplate=conf.parse.tooltip_lztemplate,
+                               tooltip_lztemplate=parse_utils.tooltip_lztemplate,
         )
     except Exception as exc:
         die("Sorry, your region request for phenocode {!r} and gene {!r} didn't work".format(phenocode, genename), exception=exc)
@@ -290,7 +346,7 @@ def gene_page(genename:str):
 
 
 
-if conf.get('download_pheno_sumstats', '') == 'secret':
+if conf.is_secret_download_pheno_sumstats():
     if app.config['SECRET_KEY'] == 'nonsecret key':
         raise PheWebError('you must set a SECRET_KEY in config.py to use download_pheno_sumstats = "secret"')
     class Hasher:
@@ -374,7 +430,7 @@ def apply_caching(response):
 
 
 ### OAUTH2
-if 'login' in conf:
+if conf.is_login_required():
     google_sign_in = GoogleSignIn(app)
 
 
@@ -393,7 +449,7 @@ if 'login' in conf:
 
     @lm.user_loader
     def load_user(id):
-        if 'whitelist' in conf.login and id not in conf.login['whitelist']:
+        if conf.get_login_allowlist() and id not in conf.get_login_allowlist():
             return None
         return User(email=id)
 
@@ -441,7 +497,7 @@ if 'login' in conf:
             flash('Authentication failed by failing to get an email address.  Please email pjvh@umich.edu')
             return redirect(url_for('.homepage'))
 
-        if 'whitelist' in conf.login and email.lower() not in conf.login['whitelist']:
+        if conf.get_login_allowlist() and email.lower() not in conf.get_login_allowlist():
             flash('Your email, {!r}, is not in the list of allowed emails.'.format(email))
             return redirect(url_for('.homepage'))
 
